<?php

declare(strict_types=1);

namespace Drupal\migrate_utils\EventSubscriber;

use Drupal\Core\State\StateInterface;
use Drupal\migrate\Event\MigrateEvents;
use Drupal\migrate\Event\MigrateImportEvent;
use Drupal\migrate\Event\MigrateRollbackEvent;
use Drupal\migrate_utils\MigrateState;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Event subscriber for migrate events.
 */
class MigrateEventSubscriber implements EventSubscriberInterface {

  public function __construct(
    protected StateInterface $state,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      MigrateEvents::PRE_IMPORT => ['onPreImport'],
      MigrateEvents::POST_IMPORT => ['onPostImport'],
      MigrateEvents::PRE_ROLLBACK => ['onPreRollback'],
      MigrateEvents::POST_ROLLBACK => ['onPostRollback'],
    ];
  }

  /**
   * Handle pre import event.
   */
  public function onPreImport(MigrateImportEvent $event): void {
    $this->state->set(MigrateState::ACTIVE_MIGRATION_KEY, $event->getMigration()->id());
    $this->state->set(MigrateState::MIGRATION_RUNNING_KEY, TRUE);
  }

  /**
   * Handle post import event.
   */
  public function onPostImport(MigrateImportEvent $event): void {
    $this->state->set(MigrateState::MIGRATION_RUNNING_KEY, FALSE);
    $this->state->delete(MigrateState::ACTIVE_MIGRATION_KEY);
  }

  /**
   * Handle pre rollback event.
   */
  public function onPreRollback(MigrateRollbackEvent $event): void {
    $this->state->set(MigrateState::ACTIVE_MIGRATION_KEY, $event->getMigration()->id());
    $this->state->set(MigrateState::MIGRATION_RUNNING_KEY, TRUE);
  }

  /**
   * Handle post rollback event.
   */
  public function onPostRollback(MigrateRollbackEvent $event): void {
    $this->state->set(MigrateState::MIGRATION_RUNNING_KEY, FALSE);
    $this->state->delete(MigrateState::ACTIVE_MIGRATION_KEY);
  }

  /**
   * Whether a migration is currently running.
   */
  public function isMigrationRunning() {
    return $this->state->get(MigrateState::MIGRATION_RUNNING_KEY, FALSE);
  }

}
